//---------------------------------------------------------------------------

#ifndef AllometryH
#define AllometryH


#include <math.h>
#include "TreePopulation.h"

/**
* Allometry Class - Version 2.2
* This handles allometric calculations for trees.  None of these functions
* check for over or underflow.
*
* Copyright 2005 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>-----------------
* <br>April 28, 2004 - Submitted as beta (LEM)
* <br>April 25, 2005 - Updated to allow user-selected allometric equations
* and updated version to 2.0 (LEM)
* <br>July 8, 2005 - Added Chapman-Richards function for crown shape;
* standardized on the name "crown" instead of "canopy"; and upgraded version
* to 2.1 (LEM)
* <br>January 25, 2006 - Added power function for saplings and upgraded version
* to 2.2 (LEM)
*/
class clAllometry {

  public:
  /**Functions for height-diam relationships.*/
  enum iHeightDiamFunction {
    standard, /**<Old standard function*/
    linear, /**<Height is a linear function of DBH*/
    reverse_linear, /**<DBH is a linear function of height*/
    power /**<Height is a power function of diam10*/
  };

  /**Functions for crown radius-diam relationships.*/
  enum iCrownRadDiamFunction {
    standard_cr, /**<Old standard function*/
    chapman_richards_cr /**<Chapman-Richards*/
  };

  /**Functions for crown height-diam relationships.*/
  enum iCrownHtDiamFunction {
    standard_ch, /**<Old standard function*/
    chapman_richards_ch /**<Chapman-Richards*/
  };

  /**
  * Constructor.  NULLs pointers.
  */
  clAllometry();

  /**
  * Destructor.  Frees memory.
  */
  ~clAllometry();

  /**
  * Reads data from the parameter file.
  * @param p_oDoc Parsed parameter file document.
  * @param p_oPop Tree population object.
  */
  void GetData(xercesc::DOMDocument *p_oDoc, clTreePopulation *p_oPop);

  /**
  * Gets the maximum tree height.
  *
  * @param iSpecies Species for which to get the maximum tree height.
  * @return Max tree height.
  * @throw Error if the species isn't valid.
  */
  float GetMaxTreeHeight(int iSpecies);

  /**
  * Calculates height according to the adult DBH-height relationship for the
  * given species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcAdultHeight( const float &fDbh, const int &iSpecies );

  /**
  * Calculates DBH according to the adult DBH-height relationship for the
  * given species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  * @throw ModelErr If the species isn't recognized or if the height is greater
  * than the maximum height for the species.
  */
  float CalcAdultDbh( float fHeight, const int &iSpecies );

  /**
  * Calculates crown radius according to the adult crown radius-DBH equation
  * for the species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Crown radius in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcAdultCrownRadius( const float &fDbh, const int &iSpecies );

  /**
  * Calculates adult crown height according to the adult allometry equation for
  * the species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Adult crown height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcAdultCrownHeight(float fHeight, const int &iSpecies);

  /**
  * Calculates height according to the sapling DBH-height relationship for the
  * given species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSaplingHeight(const float &fDbh, const int &iSpecies);

  /**
  * Calculates DBH according to the sapling DBH-height relationship for the
  * given species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  * @throw ModelErr If the species isn't recognized or if the height is greater
  * than the maximum height for the species.
  */
  float CalcSaplingDbh( float fHeight, const int &iSpecies );

  /**
  * Calculates crown radius according to the sapling DBH-crown radius
  * relationship for the given species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Crown radius in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSaplingCrownRadius( const float &fDbh, const int &iSpecies );

  /**
  * Calculates crown height according to the sapling allometry equation for the
  * species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Crown height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSaplingCrownHeight(float fHeight, const int &iSpecies);

  /**
  * Calculates height according to the seedling diam<sub>10</sub>-height
  * relationship for the given species.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSeedlingHeight( float fDiam10, const int &iSpecies );

  /**
  * Calculates diameter at 10 cm according to the seedling
  * diam<sub>10</sub>-height relationship for the given species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Diameter at 10 cm in cm.
  * @throw ModelErr If the species isn't recognized or if the height is greater
  * than the maximum height for the species.
  */
  float CalcSeedlingDiam10( float fHeight, const int &iSpecies );

  /**
  * Converts a diameter at 10 cm value to a DBH.  The equation is:
  * DBH = diam<sub>10</sub> * R where DBH is the DBH in cm, diam<sub>10</sub> is
  * the diameter at 10 cm height in cm, and R is the DBH to diam<sub>10</sub>
  * ratio.
  * @param fDiam10 Diameter at 10 cm in cm.
  * @param iSpecies Species of tree.
  * @return DBH value in cm.
  * @throw ModelErr If the species isn't recognized.
  */
  float ConvertDiam10ToDbh( float fDiam10, const int &iSpecies );

  /**
  * Converts a DBH value to a diameter at 10 cm value.  The equation is:
  * diam<sub>10</sub> = DBH / R where DBH is the DBH in cm, diam<sub>10</sub> is
  * the diameter at 10 cm height in cm, and R is the DBH to diam<sub>10</sub>
  * ratio.
  * @param fDbh DBH in cm.
  * @param iSpecies Species of tree.
  * @return Diameter at 10 cm value in cm.
  * @throw ModelErr If the species isn't recognized.
  */
  float ConvertDbhToDiam10( const float &fDbh, const int &iSpecies );

  /**
  * Gets the maximum possible crown radius.
  * @return Maximum possible crown radius, in m.
  */
  float GetMaxCrownRadius() {return m_fMaxCrownRad;};

  protected:

  /**Typedef to all allometry calculating functions*/
  typedef float (clAllometry::*Ptr2Allometry)(const float&, const int&);

  /**Function pointer array for the appropriate adult height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_AdultHeight;

  /**Function pointer array for the appropriate adult DBH-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_AdultDiam;

  /**Function pointer array for the appropriate adult crown radius-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_AdultCrownRad;

  /**Function pointer array for the appropriate adult crown height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_AdultCrownHeight;

  /**Function pointer array for the appropriate sapling height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SaplingHeight;

  /**Function pointer array for the appropriate sapling DBH-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SaplingDiam;

  /**Function pointer array for the appropriate sapling crown radius-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SaplingCrownRad;

  /**Function pointer array for the appropriate sapling crown height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SaplingCrownHeight;

  /**Function pointer array for the appropriate seedling height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SeedlingHeight;

  /**Function pointer array for the appropriate seedling diam10-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SeedlingDiam;

  /**Tree maximum height for each species.  Array size is total # species.
  * Values are required for all species.*/
  float *mp_fMaxTreeHeight;

  /**Asymptotic crown radius.  Array size is total # species.*/
  float *mp_fAsympCrownRad;

  /**Crown radius exponent.  Array size is total # species.*/
  float *mp_fCrownRadExp;

  /**Slope of DBH to diameter at 10 cm.  For converting from one to the other.
  * Array size is total # species.*/
  float *mp_fDbhToDiam10Slope;

  /**Intercept of DBH to diameter at 10 cm.  For converting from one to the
  * other. Array size is total # species.*/
  float *mp_fDbhToDiam10Intercept;

  /**Asymptotic crown height.  Array size is total # species.*/
  float *mp_fAsympCrownHeight;

  /**Crown height exponent.  Array size is total # species.*/
  float *mp_fCrownHeightExp;

  /**Slope of the height-diameter at 10 cm conversion relationship.  Array size
  * is total # species.*/
  float *mp_fSlopeHeightDiam10;

  /**Slope of asymptotic height.  Array size is total # species.*/
  float *mp_fSlopeAsympHeight;

  /**Slope of adult linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fAdultLinearSlope;

  /**Intercept of adult linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fAdultLinearIntercept;

  /**Slope of adult reverse linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fAdultReverseLinearSlope;

  /**Intercept of adult reverse linear height-dbh equation.  Array size is
  * total # species.*/
  float *mp_fAdultReverseLinearIntercept;

  /**Slope of sapling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSaplingLinearSlope;

  /**Intercept of sapling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSaplingLinearIntercept;

  /**Slope of sapling reverse linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSaplingReverseLinearSlope;

  /**Intercept of sapling reverse linear height-dbh equation.  Array size is
  * total # species.*/
  float *mp_fSaplingReverseLinearIntercept;

  /**Slope of seedling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSeedlingLinearSlope;

  /**Intercept of seedling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSeedlingLinearIntercept;

  /**Slope of seedling reverse linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSeedlingReverseLinearSlope;

  /**Intercept of seedling reverse linear height-dbh equation.  Array size is
  * total # species.*/
  float *mp_fSeedlingReverseLinearIntercept;

  /**Crown radius intercept for the Chapman-Richards equation.  Array size is
  * total # species.*/
  float *mp_fCRCrownRadIntercept;

  /**Asymptotic crown radius for the Chapman-Richards equation.  Array size is
  * total # species.*/
  float *mp_fCRAsympCrownRad;

  /**Chapman-Richards crown radius equation shape parameter 1.  Array size is
  * total # species.*/
  float *mp_fCRCrownRadShape1;

  /**Chapman-Richards crown radius equation shape parameter 2.  Array size is
  * total # species.*/
  float *mp_fCRCrownRadShape2;

  /**Crown height intercept for the Chapman-Richards equation.  Array size is
  * total # species.*/
  float *mp_fCRCrownHtIntercept;

  /**Asymptotic crown height for the Chapman-Richards equation.  Array size is
  * total # species.*/
  float *mp_fCRAsympCrownHt;

  /**Chapman-Richards crown height equation shape parameter 1.  Array size is
  * total # species.*/
  float *mp_fCRCrownHtShape1;

  /**Chapman-Richards crown height equation shape parameter 2.  Array size is
  * total # species.*/
  float *mp_fCRCrownHtShape2;

  /**Power function a parameter.  Array size is total # species.*/
  float *mp_fPowerA;

  /**Power function b parameter (exponent).  Array size is total # species.*/
  float *mp_fPowerExpB;

  /**Max crown radius for standard equation*/
  float m_fMaxStdCrownRad;

  /**Maximum possible crown radius*/
  float m_fMaxCrownRad;

  /**Total number of species*/
  int m_iNumSpecies;


  /**
  * Calculates height from DBH according to the standard sapling-adult
  * allometry function.  This equation is:
  * <br>Height = 1.35 + (H<sub>1</sub> - 1.35)*(1 - e<sup>-B * DBH</sup>)
  * <br>
  * where
  * Height is height in meters, H<sub>1</sub> is max tree height, B is slope
  * of asymptotic height, and DBH is tree DBH in cm.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcStandardSapAdultHeight(const float &fDbh, const int &iSpecies);

  /**
  * Calculates height from DBH according to the linear function.  This equation
  * is:
  * <br>Height = a + b * DBH
  * <br>
  * where b is the adult linear slope and a is the adult linear intercept.
  * The height is limited to the maximum height for the species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcLinearAdultHeight(const float &fDbh, const int &iSpecies );

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>Height = (DBH - a) / b
  * <br>
  * where b is the adult reverse linear slope and a is the adult reverse linear
  * intercept.  The height is limited to the maximum height for the species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcReverseLinearAdultHeight(const float &fDbh, const int &iSpecies );

  /**
  * Calculates height from DBH according to the linear function.  This equation
  * is:
  * <br>Height = a + b * DBH
  * <br>
  * where b is the sapling linear slope and a is the sapling linear intercept.
  * The height is limited to the maximum height for the species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcLinearSaplingHeight(const float &fDbh, const int &iSpecies );

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>Height = (DBH - a) / b
  * <br>
  * where b is the sapling reverse linear slope and a is the sapling reverse
  * linear intercept.  The height is limited to the maximum height for the
  * species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcReverseLinearSaplingHeight(const float &fDbh, const int &iSpecies );

  /**
  * Calculates height from DBH according to the power function.  This
  * equation is:
  * <br>Height = a * D <sup>b</sup>
  * <br>
  * where D is in fact Diam10 - this will convert back and forth with DBH.
  * The height is limited to the maximum height for the species.  If fDbh is
  * negative or 0, this returns the minimum height (0.001).
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcPowerSaplingHeight(const float &fDbh, const int &iSpecies );

  /**
  * Calculates height according to standard seedling diam10-height allometry
  * equation.  This equation is:
  * <br>height = 0.1 + 30*(1 - e<sup>-alpha * diam<sub>10</sub>)</sup>)<br>
  * where height is tree height in m, alpha is the slope of the height -
  * diam<sub>10</sub> relationship, and diam<sub>10</sub> is the tree's diameter
  * at 10 cm height.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcStandardSeedlingHeight( const float &fDiam10, const int &iSpecies );

  /**
  * Calculates height from diameter at 10 cm according to the linear function.
  * This equation is:
  * <br>Height = a + b * diam<sub>10</sub>
  * <br>
  * where b is the seedling linear slope and a is the seedling linear
  * intercept.  The height is limited to the maximum height for the species.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcLinearSeedlingHeight(const float &fDiam10, const int &iSpecies );

  /**
  * Calculates height from diameter at 10 cm according to the reverse linear
  * function. This equation is:
  * <br>Height = (diam<sub>10</sub> - a) / b
  * <br>
  * where b is the seedling reverse linear slope and a is the seedling reverse
  * linear intercept.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcReverseLinearSeedlingHeight(const float &fDiam10, const int &iSpecies );

  /**
  * Calculates DBH from height according to the standard sapling-adult
  * allometry function.  This equation (which is just the height equation
  * switched around) is:
  * DBH = log(1-((Height-1.35)/(H<sub>1</sub>-1.35)))/-B where
  * Height is height in meters, H<sub>1</sub> is max tree height, B is slope
  * of asymptotic height, and DBH is tree DBH in cm.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcStandardSapAdultDbh( const float &fHeight, const int &iSpecies );

  /**
  * Calculates DBH from height according to the linear function.  This equation
  * is:
  * <br>DBH = (Height - a) / b
  * <br>
  * where b is the adult linear slope and a is the adult linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcLinearAdultDbh(const float &fHeight, const int &iSpecies );

  /**
  * Calculates DBH from height according to the reverse linear function.  This
  * equation is:
  * <br>DBH = a + b * Height
  * <br>
  * where b is the adult reverse linear slope and a is the adult reverse linear
  * intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcReverseLinearAdultDbh(const float &fHeight, const int &iSpecies );

  /**
  * Calculates DBH from height according to the linear function.  This equation
  * is:
  * <br>DBH = (Height - a) / b
  * <br>
  * where b is the sapling linear slope and a is the sapling linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcLinearSaplingDbh(const float &fHeight, const int &iSpecies );

  /**
  * Calculates DBH from height according to the reverse linear function.  This
  * equation is:
  * <br>DBH = a + b * Height
  * <br>
  * where b is the sapling reverse linear slope and a is the sapling reverse
  * linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcReverseLinearSaplingDbh(const float &fHeight, const int &iSpecies );

  /**
  * Calculates DBH from height according to the reverse linear function.  This
  * equation is:
  * <br>D = (H / a) <sup>1/b</sup>
  * <br>
  * d is the diam10.  Then the conversion to DBH is made.  If fHeight is less
  * than or approximately equal to 0, this returns 0.001.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcPowerSaplingDbh(const float &fHeight, const int &iSpecies );

  /**
  * Calculates diameter at 10 cm according to the standard seedling height -
  * diam10 allometry equation (which is just the height equation switched
  * around).  The equation is:
  * diam<sub>10</sub> = (log(1-((height-0.1)/30)))/-alpha where
  * height is tree height in m, alpha is the slope of the height -
  * diam<sub>10</sub> relationship, and diam<sub>10</sub> is the tree's diameter
  * at 10 cm height.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Diameter at 10 cm in cm.
  */
  float CalcStandardSeedlingDiam10( const float &fHeight, const int &iSpecies );

  /**
  * Calculates height from diameter at 10 according to the linear function.
  * This equation is:
  * <br>diam<sub>10</sub> = (Height - a) / b
  * <br>
  * where b is the seedling linear slope and a is the seedling linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcLinearSeedlingDiam10(const float &fHeight, const int &iSpecies );

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>diam<sub>10</sub> = a + b * Height
  * <br>
  * where b is the seedling reverse linear slope and a is the seedling reverse
  * linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcReverseLinearSeedlingDiam10(const float &fHeight, const int &iSpecies );

  /**
  * Calculates crown radius according to the standard equation.  This
  * equation is: crad = C<sub>1</sub> * DBH <sup>a</sup> where
  * crad is the crown radius in meters, C<sub>1</sub> is the asymptotic
  * crown radius, DBH is the DBH of the tree in cm, and a is the crown radius
  * exponent.  The value of crad cannot exceed 10 m.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Crown radius in meters, up to a maximum value of
  * m_fMaxStdCrownRad.
  */
  float CalcStandardSapAdultCrownRad(const float &fDbh, const int &iSpecies);

  /**
  * Calculates adult crown radius using the Chapman-Richards equation.  This
  * equation is:  CR = i + a * (1 - exp(-b * DBH)) <sup>c</sup> where CR =
  * crown radius in meters, i = crown radius intercept in meters, a =
  * asymptotic crown radius (in meters), b = shape parameter 1, c = shape
  * parameter 2, DBH = DBH of the tree in cm.  Since this equation is already
  * self-limiting, it's not subjected to the limit of 10 m like the standard
  * equation is.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Crown radius in meters.
  */
  float CalcChapRichSapAdultCrownRad(const float &fDbh, const int &iSpecies);

  /**
  * Calculates crown height according to the standard equation. This
  * equation is: CH = C<sub>2</sub> * Height <sup>a</sup> where
  * CH is the height of the crown in meters, C<sub>2</sub> is the asymptotic
  * crown height, Height is the tree's height in meters, and a is the crown
  * height exponent.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Adult crown height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcStandardSapAdultCrownHeight(const float &fHeight, const int &iSpecies);

  /**
  * Calculates crown height using the Chapman-Richards equation.  This
  * equation is:  CH = i + a * (1 - exp(-b * H)) <sup>c</sup> where CR =
  * crown radius in meters, i = crown radius intercept in meters, a =
  * asymptotic crown radius (in meters), b = shape parameter 1, c = shape
  * parameter 2, DBH = DBH of the tree in cm.  Since this equation is already
  * self-limiting, it's not subjected to the limit of m_fMaxStdCrownRad like
  * the standard equation is.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Crown radius in meters.
  */
  float CalcChapRichSapAdultCrownHeight(const float &fHeight, const int &iSpecies);
}; //end of class clAllometry


//---------------------------------------------------------------------------
#endif
