//---------------------------------------------------------------------------
// NeighborhoodSeedPredation
//---------------------------------------------------------------------------
#if !defined(NeighborhoodSeedPredation_H)
  #define NeighborhoodSeedPredation_H

#include "BehaviorBase.h"

class clGridBase;
class clSimManager;

/**
* Neighborhood Seed Predation - version 1.0
*
* This behavior reduces the number of seeds by simulating seed predation. Rate
* of offtake is a function of neighborhood composition and masting.
* 
* Neighborhood is assessed by computing a PCA score using the basal area of
* trees in a particular radius around each seed grid cell. This is done as
* follows:
* @htmlonly
* <br><center>X = &Sigma; p<sub>n</sub> * RBA <sub>n</sub></center>
* @endhtmlonly
* where:</p>
* <ul>
* <li>X is the neighborhood PCA score, summed over species n=1 to N total 
* species</li>
* <li>p<sub>n</sub> is the loading factor for species n</li>
* <li>RBA<sub>n</sub> is the relative basal area of species n</li>
* </ul>
*
* Masting is assessed by counting the seeds to which this behavior applies and
* comparing it to a density per timestep threshold. Above the threshold is
* masting; below is not. The only difference between masting and non-masting
* timesteps is the parameters used.
* 
* The proportion of seed removed is calculated as:
* <br><center>Y = A * B<sup>(X - C)<sup>2</sup></sup></center>
* where A, B, and C are parameters that have different values in masting and
* non-masting years.
*
* A fatal error is thrown if a disperse behavior is not also defined for the
* run.
*
* This class's namestring and parameter call string are both
* "Neighborhood Seed Predation".
* <br>Copyright 2007 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>-----------------
* <br>May 21, 2007 - Created (LEM)
*/
class clNeighborhoodSeedPredation : virtual public clBehaviorBase {
//note: need the virtual keyword to avoid base class ambiguity.

 public:

  /**
  * Constructor.
  *
  * @param p_oSimManager Sim Manager object.
  */
  clNeighborhoodSeedPredation(clSimManager *p_oSimManager);

  /**
  * Destructor.
  */
  ~clNeighborhoodSeedPredation();

  /**
  * Does setup. This reads in values from the parameter file and gets pointers
  * to the grid.
  *
  * @param p_oDoc DOM tree of parsed input file.
  * @throws modelErr if any of the following are negative: neighborhood radius,
  * min neighbor DBH, masting threshold; or if the seed grid is not found. 
  */
  void GetData(xercesc::DOMDocument *p_oDoc);

  /**
  * Performs neighborhood seed predation. First, for the whole plot, this 
  * assesses whether masting has occurred by comparing the density of seeds
  * of species to which this behavior applies to the threshold. Then for each
  * cell, the neighborhood PCA score and the proportion to remove is calculated.
  * A random round is used to take care of fractional parts of seeds.
  */
  void Action();

  protected:

  /**Pointer to the "Dispersed Seeds" grid created by disperse behaviors.*/
  clGridBase *mp_oSeedGrid;

  /**Non-masting "A" parameter.  Array size is number of species to which this
   * behavior applies.*/
  float *mp_fNonMastingA;
  
  /**Non-masting "B" parameter.  Array size is number of species to which this
   * behavior applies.*/
  float *mp_fNonMastingB;
  
  /**Non-masting "C" parameter.  Array size is number of species to which this
   * behavior applies.*/
  float *mp_fNonMastingC;
  
  /**Masting "A" parameter.  Array size is number of species to which this
   * behavior applies.*/
  float *mp_fMastingA;
  
  /**Masting "B" parameter.  Array size is number of species to which this
   * behavior applies.*/
  float *mp_fMastingB;
  
  /**Masting "C" parameter.  Array size is number of species to which this
   * behavior applies.*/
  float *mp_fMastingC;
  
  /**PCA masting loading factor. Array size is total number of species.*/
  float *mp_fMastingLoadingFactor;
  
  /**PCA non-masting loading factor. Array size is total number of species.*/
  float *mp_fNonMastingLoadingFactor;
  
  /**Data member codes for seed grid for number of seeds.  Array size is number
   * of species to which this behavior applies.*/
  short int *mp_iSeedGridCode;
  
  /** For accessing the other arrays.*/
  short int *mp_iIndexes;

  /**Area of the plot, in square meters.*/
  float m_fPlotArea;
  
  /**Radius of neighborhoods.*/
  float m_fRadius;

  /**Threshold of seeds per square meter for a masting timestep. This comes as
   * an annual amount in the parameter file and is converted to a timestep
   * amount.*/
  float m_fMastingThreshold;

  /**Minimum neighbor DBH.*/
  float m_fMinNeighDBH;
};
//---------------------------------------------------------------------------
#endif 


