//---------------------------------------------------------------------------
#include "RelativeGrowth.h"
#include "GrowthOrg.h"
#include "Tree.h"
#include "SimManager.h"
#include "ParsingFunctions.h"
#include <math.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// October 26, 2005 - Added new array (LEM)
//////////////////////////////////////////////////////////////////////////////
clRelativeGrowth::clRelativeGrowth(clSimManager *p_oSimManager) :
  clWorkerBase( p_oSimManager ), clBehaviorBase( p_oSimManager ),
  clGrowthBase( p_oSimManager ), clMichMenBase(p_oSimManager)
 {
    strcpy(m_cNameString, "relativegrowthshell");
    //Version 1.1
    m_fVersionNumber = 1.1;
    m_fMinimumVersionNumber = 1.1;
    mp_fDiamExp = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Destructor
// Edit history:
// -------------
// October 26, 2005 - Created, in order to delete the new array (LEM)
//////////////////////////////////////////////////////////////////////////////
clRelativeGrowth::~clRelativeGrowth() {
  delete[] mp_fDiamExp;
}

//////////////////////////////////////////////////////////////////////////////
// SetNameData()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
//////////////////////////////////////////////////////////////////////////////
void clRelativeGrowth::SetNameData( char * cNameString )
{
  try
  {

    //Check the string passed and set the flags accordingly
    if ( strcmp( "relradialgrowth", cNameString ) == 0 )
    {
      m_bConstRadialLimited = true;
      m_bConstBasalAreaLimited = false;
      m_iGrowthMethod = diameter_auto;
    }
    else if ( strcmp( "relbagrowth", cNameString ) == 0 )
    {
      m_bConstRadialLimited = false;
      m_bConstBasalAreaLimited = true;
      m_iGrowthMethod = diameter_auto;
    }
    else if ( strcmp( "relunlimgrowth", cNameString ) == 0 )
    {
      m_bConstRadialLimited = false;
      m_bConstBasalAreaLimited = false;
      m_iGrowthMethod = diameter_auto;
    }
    else if ( strcmp( "relradialgrowth diam only", cNameString ) == 0 )
    {
      m_bConstRadialLimited = true;
      m_bConstBasalAreaLimited = false;
      m_iGrowthMethod = diameter_only;
    }
    else if ( strcmp( "relbagrowth diam only", cNameString ) == 0 )
    {
      m_bConstRadialLimited = false;
      m_bConstBasalAreaLimited = true;
      m_iGrowthMethod = diameter_only;
    }
    else if ( strcmp( "relunlimgrowth diam only", cNameString ) == 0 )
    {
      m_bConstRadialLimited = false;
      m_bConstBasalAreaLimited = false;
      m_iGrowthMethod = diameter_only;
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clRelativeGrowth::SetNameData" );
    throw( stcErr );
  }
}


//////////////////////////////////////////////////////////////////////////////
// DoShellSetup()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// August 27, 2004 - Made light required (LEM)
//////////////////////////////////////////////////////////////////////////////
void clRelativeGrowth::DoShellSetup( DOMDocument * p_oDoc )
{
  try
  {
    clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );
    DOMElement * p_oElement = p_oDoc->getDocumentElement();
    floatVal * p_fTempValues; //for getting species-specific values
    short int iNumSpecies = mp_oGrowthOrg->GetNumberOfSpecies(), i;

    //Set up our floatVal array that will extract values only for the species
    //assigned to this behavior
    p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      p_fTempValues[i].code = mp_iWhatSpecies[i];

    m_fNumberYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep();

    //Declare the arrays we'd like read
    mp_fAsympDiamGrowth = new float[iNumSpecies];
    mp_fSlopeGrowthResponse = new float[iNumSpecies];
    mp_fDiamExp = new float[iNumSpecies];

    if ( m_bConstRadialLimited )
    {
      mp_fAdultConstRadInc = new float[iNumSpecies];
    }

    if ( m_bConstBasalAreaLimited )
    {
      mp_fAdultConstBAInc = new float[iNumSpecies];
    }

    //Read the base variables
    GetParameterFileData( p_oDoc );

    //Get the diameter exponent
    FillSpeciesSpecificValue( p_oElement, "gr_relGrowthDiamExp", "gr_rgdeVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fDiamExp[p_fTempValues[i].code] = p_fTempValues[i].val;


    //Make sure all species/type combos have "Light" registered
    for ( int i = 0; i < m_iNumSpeciesTypeCombos; i++ )
    {
      if ( -1 == mp_oGrowthOrg->GetLightCode(mp_whatSpeciesTypeCombos[i].iSpecies,
                                           mp_whatSpeciesTypeCombos[i].iType ) )
      {
        modelErr stcErr;
        strcpy( stcErr.cFunction, "clRelativeGrowth::DoShellSetup" );
        sprintf( stcErr.cMoreInfo, "%s%d%s%d%s",
                 "Type/species combo species=",
                 mp_whatSpeciesTypeCombos[i].iSpecies,
                 " type=", mp_whatSpeciesTypeCombos[i].iType,
                 " does not have a required light behavior." );
        stcErr.iErrorCode = BAD_DATA;
        throw( stcErr );
      }
    }

    delete[] p_fTempValues;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clRelativeGrowth::DoShellSetup" );
    throw( stcErr );
  }
}


//////////////////////////////////////////////////////////////////////////////
// CalcDiameterGrowthValue()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
//////////////////////////////////////////////////////////////////////////////
float clRelativeGrowth::CalcDiameterGrowthValue( clTree * p_oTree, clTreePopulation * p_oPop, float fHeightGrowth )
{
  try
  {
    float fAnnualRelativeGrowth, //amount of annual relative growth
         fGli, //tree's gli value
         fDiam, //tree's diameter value
         fCompoundRelativeGrowth, //relative growth compounded to a complete
         //timestep
         fAmountDiamIncrease; //amount by which the tree's diameter will
    //increase
    short int iSpecies = p_oTree->GetSpecies(), iType = p_oTree->GetType(),
         iLightCode = mp_oGrowthOrg->GetLightCode( iSpecies, iType );

    //***************************************
    // Calculate the amount of growth
    //***************************************

    //Get the gli value for this tree
    p_oTree->GetValue( iLightCode, & fGli );

    //Get the appropriate diameter for this tree
    p_oTree->GetValue( mp_oGrowthOrg->GetDiamCode( iSpecies, iType ), & fDiam );

    //Get the value of the Michaelis-Menton function
    fAnnualRelativeGrowth = CalculateMichaelisMentonFunction( iSpecies, fGli );

    //Compound the relative growth over the number of years/time step
    fCompoundRelativeGrowth = pow( 1.0 + fAnnualRelativeGrowth, m_fNumberYearsPerTimestep );

    //Calculate amount of diameter increase based on the tree's diameter
    fAmountDiamIncrease = pow(fDiam, mp_fDiamExp[iSpecies]) * ( fCompoundRelativeGrowth - 1.0 );

    fAmountDiamIncrease = ApplyGrowthLimits( iSpecies, fAmountDiamIncrease, fDiam );

    return fAmountDiamIncrease;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clRelativeGrowth::CalcGrowthValue" );
    throw( stcErr );
  }
}
//---------------------------------------------------------------------------
