//---------------------------------------------------------------------------
#include "StochasticBiLevelMortality.h"
#include "MortalityOrg.h"
#include "GridBase.h"
#include "ModelMath.h"
#include "ParsingFunctions.h"
#include "TreePopulation.h"
#include "SimManager.h"
#include <math.h>

////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// October 31, 2005 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
clStochasticBiLevelMortality::clStochasticBiLevelMortality( clSimManager * p_oSimManager ) : 
     clWorkerBase( p_oSimManager ), clBehaviorBase( p_oSimManager ), 
     clMortalityBase( p_oSimManager )
     {
       try
       {

         strcpy( m_cNameString, "stochastic bilevel mortshell" );

         mp_oStormLight = NULL;
         mp_fLoLightMortProb = NULL;
         mp_fHiLightMortProb = NULL;
         mp_fHiLightThreshold = NULL;
         mp_iIndexes = NULL;
       }
       catch ( modelErr & err )
       {
         throw( err );
       }
       catch ( modelMsg & msg )
       {
         throw( msg );
       } //non-fatal error
       catch ( ... )
       {
         modelErr stcErr;
         stcErr.iErrorCode = UNKNOWN;
         strcpy( stcErr.cFunction, "clStochasticBiLevelMortality::clStochasticBiLevelMortality" );
         throw( stcErr );
       }
}

////////////////////////////////////////////////////////////////////////////
// Destructor
// Edit history:
// -------------
// October 31, 2005 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
clStochasticBiLevelMortality::~clStochasticBiLevelMortality()
{
  delete[] mp_fLoLightMortProb;
  delete[] mp_fHiLightMortProb;
  delete[] mp_fHiLightThreshold;
  delete[] mp_iIndexes;
}

////////////////////////////////////////////////////////////////////////////
// DoShellSetup()
// Edit history:
// -------------
// October 31, 2005 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
void clStochasticBiLevelMortality::DoShellSetup( xercesc::DOMDocument * p_oDoc )
{
  try
  {
    mp_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );

    DOMElement * p_oElement = p_oDoc->getDocumentElement();
    floatVal * p_fTempValues; //for getting species-specific values
    float fYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep();
    short int iNumSpecies = mp_oPop->GetNumberOfSpecies(), i;

    //Declare the arrays we'd like read
    mp_fLoLightMortProb = new float[m_iNumBehaviorSpecies];
    mp_fHiLightMortProb = new float[m_iNumBehaviorSpecies];
    mp_fHiLightThreshold = new float[m_iNumBehaviorSpecies];
    mp_iIndexes = new int[iNumSpecies];

    //Set up the array indexes
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_iIndexes[mp_iWhatSpecies[i]] = i;

    //See if we can find the storm light grid
    mp_oStormLight = mp_oSimManager->GetGridObject( "Storm Light" );
    if ( NULL != mp_oStormLight )
    {
      m_iLightCode = mp_oStormLight->GetFloatDataCode( "Light" );
    }
    else
    {
      modelErr stcErr;
      stcErr.iErrorCode = CANT_FIND_OBJECT;
      strcpy( stcErr.cFunction, "clStochasticBiLevelMortality::DoShellSetup" );
      strcpy( stcErr.cMoreInfo, "\"Storm Light\" grid is missing." );
      throw( stcErr );
    }

    //Declare the species-specific temp array and pre-load with the species
    //that this behavior affects
    p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      p_fTempValues[i].code = mp_iWhatSpecies[i];

    //Low-light mortality probability
    FillSpeciesSpecificValue( p_oElement, "mo_stochBilevLoLiteMortProb", "mo_sbllmpVal", p_fTempValues,
         m_iNumBehaviorSpecies, mp_oPop, true );
    //Transfer values to our permanent array, making sure all values are
    //between 0 and 1 and translating to a timestep probability
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    {
      if ( 0 > p_fTempValues[i].val || 1 < p_fTempValues[i].val )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clStochasticBiLevelMortality::DoShellSetup" );
        strcpy( stcErr.cMoreInfo, "All values for probability of mortality must be between 0 and 1." );
        throw( stcErr );
      }
      mp_fLoLightMortProb[mp_iIndexes[p_fTempValues[i].code]] = 1 - pow( 1 - p_fTempValues[i].val, fYearsPerTimestep );
    }

    //Translate the values

    //High-light mortality probability
    FillSpeciesSpecificValue( p_oElement, "mo_stochBilevHiLiteMortProb", "mo_sbhlmpVal", p_fTempValues,
         m_iNumBehaviorSpecies, mp_oPop, true );
    //Transfer values to our permanent array, making sure all values are
    //between 0 and 1 and translating to a timestep probability
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    {
      if ( 0 > p_fTempValues[i].val || 1 < p_fTempValues[i].val )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clStochasticBiLevelMortality::DoShellSetup" );
        strcpy( stcErr.cMoreInfo, "All values for probability of mortality must be between 0 and 1." );
        throw( stcErr );
      }
      mp_fHiLightMortProb[mp_iIndexes[p_fTempValues[i].code]] = 1 - pow( 1 - p_fTempValues[i].val, fYearsPerTimestep );
    }

    //High-light threshold
    FillSpeciesSpecificValue( p_oElement, "mo_stochBilevHiLiteThreshold", "mo_sbhltVal", p_fTempValues,
         m_iNumBehaviorSpecies, mp_oPop, true );
    //Transfer values to our permanent array, making sure all values are
    //between 0 and 100
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    {
      if ( p_fTempValues[i].val < 0 || p_fTempValues[i].val > 100 )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clStochasticBiLevelMortality::DoShellSetup" );
        strcpy( stcErr.cMoreInfo, "All values in high-light growth threshold must be between 0 and 100." );
        throw( stcErr );
      }
      mp_fHiLightThreshold[mp_iIndexes[p_fTempValues[i].code]] = p_fTempValues[i].val;
    }

    delete[] p_fTempValues;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clStochasticBiLevelMortality::DoShellSetup" );
    throw( stcErr );
  }
}

///////////////////////////////////////////////////////////////////////////
// DoMort()
// Edit history:
// -------------
// October 31, 2005 - Created (LEM)
///////////////////////////////////////////////////////////////////////////
bool clStochasticBiLevelMortality::DoMort( clTree * p_oTree, const float & fDiam, const short int & iSpecies )
{
  try
  {

    float fLightLevel, fX, fY;
    int iTp = p_oTree->GetType();

    //Get the storm light level at the tree's location
    p_oTree->GetValue( mp_oPop->GetXCode( iSpecies, iTp ), & fX );
    p_oTree->GetValue( mp_oPop->GetYCode( iSpecies, iTp ), & fY );

    mp_oStormLight->GetValueAtPoint( fX, fY, m_iLightCode, & fLightLevel );

    //Calculate the function value according to the light level
    if ( fLightLevel < mp_fHiLightThreshold[mp_iIndexes[iSpecies]] ) {
      if ( clModelMath::GetRand() < mp_fLoLightMortProb[mp_iIndexes[iSpecies]] )
        return true;
      else
        return false;

    }
    else
    {
      if ( clModelMath::GetRand() < mp_fHiLightMortProb[mp_iIndexes[iSpecies]] )
        return true;
      else
        return false;
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clStochasticBiLevelMortality::DoMort" );
    throw( stcErr );
  }
}
