//---------------------------------------------------------------------------

#ifndef StochasticBiLevelMortalityH
#define StochasticBiLevelMortalityH
//---------------------------------------------------------------------------
#include "MortalityBase.h"

class clGridBase;
class clTreePopulation;
/**
* Stochastic Bi-Level Mortality - Version 1.0
*
* This evaluates mortality at a constant rate, with a separate rate at high and
* low light for each species.  Trees are chosen at random to die.
*
* Light levels come from the "Storm Light" grid object produced by
* clStormLight.
*
* This class's namestring is "stochastic bilevel mortshell".  The parameter
* file call string is "Stochastic Bi-Level Mortality".
*
* Copyright 2005 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>-----------------
* <br>October 31, 2005 - Created (LEM)
*/
class clStochasticBiLevelMortality : virtual public clMortalityBase {
//note: need the virtual keyword to avoid base class ambiguity.

  public:

  /**
  * Constructor.
  *
  * @param p_oSimManager Sim Manager object.
  */
  clStochasticBiLevelMortality(clSimManager *p_oSimManager);

  /**
  * Destructor.
  */
  ~clStochasticBiLevelMortality();

  /**
  * Reads in values from the parameter file.
  *
  * @param p_oDoc DOM tree of parsed input file.
  * @throws modelErr if the "Storm Light" grid is missing, or if any mortality
  * rate is not between 0 and 1.
  */
  void DoShellSetup(xercesc::DOMDocument *p_oDoc);

  /**
  * Calculates mortality.  This retrieves the light level in the tree's grid
  * cell.  If it is above the threshold, the high-light mortality probability
  * is used.  If it is below the threshold, the low-light probability is used.
  * A random number is compared to the appropriate rate to see if the tree
  * lives or dies.
  *
  * @param p_oTree Tree being evaluated
  * @param fDbh Tree's DBH
  * @param iSpecies Species of the tree being evaluated
  * @return True if the tree is to die, false if it lives.
  */
  bool DoMort(clTree *p_oTree, const float &fDbh, const short int &iSpecies);

  protected:

  /**"Storm Light" grid object*/
  clGridBase *mp_oStormLight;

  /**Tree population - for getting data codes*/
  clTreePopulation *mp_oPop;

  /**Probability of mortality at low light - sized number of behavior species.
  * This value comes from the parameter file as an annual probability, which
  * is transformed in DoShellSetup() to a timestep probability.*/
  float *mp_fLoLightMortProb;

  /**Probability of mortality at low light - sized number of behavior species.
  * This value comes from the parameter file as an annual probability, which
  * is transformed in DoShellSetup() to a timestep probability.*/
  float *mp_fHiLightMortProb;

  /**Threshold between low light and high light mortality, as a value between 0
  * and 100 - sized number of behavior species*/
  float *mp_fHiLightThreshold;

  /**To help access the other arrays*/
  int *mp_iIndexes;

  /**Code for the "Light" data member of the "Storm Light" grid*/
  int m_iLightCode;

};
//---------------------------------------------------------------------------
#endif
